# Generated by Django 3.1 on 2020-09-08 17:37

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('django_notifications', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='notificationqueue',
            name='send_after',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
    ]
