# Generated by Django 3.1 on 2020-09-07 19:30

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='NotificationQueue',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('attempts_number', models.PositiveIntegerField(default=0)),
                ('backend', models.CharField(max_length=255)),
                ('custom_notification_id', models.CharField(max_length=255)),
                ('data', models.JSONField()),
                ('queue', models.PositiveSmallIntegerField(choices=[(1, 'Standard queue'), (2, 'Error queue')], default=1)),
                ('created', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'unique_together': {('custom_notification_id', 'backend')},
            },
        ),
    ]
